#include <main.h>
#include <I2C_Functions.h>

void InitI2C_v(void)
{
  /* Enable internal pull-up on PC0, PC1. Comment if you have external pull ups
   * Configure several PINxCTRL registers at the same time */
  PORTCFG.MPCMASK = 0x03;

  /* Enable pull-up to get a defined level on the I2C lines */
  TWI_PORT.PIN0CTRL = (TWI_PORT.PIN0CTRL & ~PORT_OPC_gm) | PORT_OPC_PULLUP_gc;

  /* Enable TWI Low-Level Interrupt for RX/TX and set to master mode */
//  TWI.MASTER.CTRLA = TWI_MASTER_INTLVL_LO_gc | TWI_MASTER_RIEN_bm | TWI_MASTER_WIEN_bm | TWI_MASTER_ENABLE_bm;
  TWI.MASTER.CTRLA = TWI_MASTER_ENABLE_bm;

  /* Enable I2C inactive timeout */
  TWI.MASTER.CTRLB = TWI_MASTER_TIMEOUT0_bm | TWI_MASTER_TIMEOUT1_bm;

  /* Set transfer rate */
  TWI.MASTER.BAUD = (u8) TWI_BAUDSETTING;

  /* Force bus into idle state */
  TWI.MASTER.STATUS = TWI_MASTER_BUSSTATE_IDLE_gc;
}

/* Send n bytes to specified address to I2C bus */
bool WriteI2C_b(u8 Address_u8, u8 NumberBytes_u8, u8 * Data_p)
{
  u8 Counter_u8;
  u8 WriteAddress = Address_u8 << 1;

  /* Send address */
  TWI.MASTER.ADDR = WriteAddress;

  /* Wait until byte transferred */
  while (!(TWI.MASTER.STATUS & TWI_MASTER_WIF_bm));

  if (TWI.MASTER.STATUS & TWI_MASTER_RXACK_bm)
  {
    /* Stop transfer because NACK received */
    TWI.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;

    return false;
  }

  for (Counter_u8 = 0; Counter_u8 < NumberBytes_u8; Counter_u8++)
  {
    /* Send data */
    TWI.MASTER.DATA = *(Data_p + Counter_u8);

    /* Wait until byte transferred */
    while (!(TWI.MASTER.STATUS & TWI_MASTER_WIF_bm));

    if (TWI.MASTER.STATUS & TWI_MASTER_RXACK_bm)
    {
      /* Stop transfer because NACK received */
      TWI.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;

      return false;
    }
  }

  TWI.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;

  return true;
}

/* Receive n bytes for specified address from I2C bus */
bool ReadI2C_b(u8 Address_u8, u8 NumberBytes_u8, u8 * Data_p)
{
  u8 Counter_u8;
  u8 ReadAddress = (Address_u8 << 1) | 1;

  /* Send address */
  TWI.MASTER.ADDR = ReadAddress;

  /* Wait until byte transferred */
//  while (!(TWI.MASTER.STATUS & TWI_MASTER_WIF_bm));

  if (TWI.MASTER.STATUS & TWI_MASTER_RXACK_bm)
  {
    /* Stop transfer because NACK received */
    TWI.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;

    return false;
  }

  for (Counter_u8 = 0; Counter_u8 < NumberBytes_u8; Counter_u8++)
  {
    /* Wait until byte received */
    while (!(TWI.MASTER.STATUS & TWI_MASTER_RIF_bm));

    *(Data_p + Counter_u8) = TWI.MASTER.DATA;

    if (Counter_u8 == (NumberBytes_u8 - 1))
    {
      /* Last byte received. Send NACK */
      TWI.MASTER.CTRLC = TWI_MASTER_ACKACT_bm | TWI_MASTER_CMD_STOP_gc;
    }
    else
    {
      /* Send ACK */
      TWI.MASTER.CTRLC = TWI_MASTER_CMD_RECVTRANS_gc;
    }
  }

  return true;
}

/* TWIC Master Interrupt vector */
ISR(TWIC_TWIM_vect)
{
  u8 currentStatus_u8 = TWI.MASTER.STATUS;

  if ((currentStatus_u8 & TWI_MASTER_ARBLOST_bm) || (currentStatus_u8 & TWI_MASTER_BUSERR_bm))
  {
    /* Arbitration lost or bus error. Clear interrupt flag */
    TWI.MASTER.STATUS = currentStatus_u8 | TWI_MASTER_ARBLOST_bm;
  }
  else if (currentStatus_u8 & TWI_MASTER_WIF_bm)
  {
    /* master write interrupt. */
    if (currentStatus_u8 & TWI_MASTER_RXACK_bm)
    {
      /* Stop transfer because NACK received */
      TWI.MASTER.CTRLC = TWI_MASTER_CMD_STOP_gc;
    }

    /* Continue code */
  }
  else if (currentStatus_u8 & TWI_MASTER_RIF_bm)
  {
    /* Master read interrupt. */
  }
  else
  {
    /* Unexpected state. */
  }
}
