#include <main.h>
#include <RS232_Functions.h>

u8 TxBuf_u8[MAX_LENGTH_TXBUF], RxBuf_u8[MAX_LENGTH_RXBUF];
u8 TailTx_u8 = 0;
u8 HeadTx_u8 = 0;
u8 TailRx_u8 = 0;
u8 HeadRx_u8 = 0;
u8 Newline_u8 = NEWLINE_LF;
bool OverrunTx_b = 0;
bool OverrunRx_b = 0;
bool TxInProgress_b = 0;
bool ModeChange_b = 0;

void InitRS232_v(void)
{
  /* This PORT setting is only valid to USARTC0 if other USART is used a different PORT and/or pins is used.
   * PIN3 (TXD0) as output. */
  USART_PORT.DIRSET = PIN3_bm;

  /* PIN2 (RXD0) as input. */
  USART_PORT.DIRCLR = PIN2_bm;

  /* 8N1 */
  USART.CTRLC = USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc;

  /* Baudrate select = I/O clock frequency / (16 * Baudrate) - 1
   * Equation only valid for BSCALE = 0, XMega AU manual P.293 table 23-1 */
  USART.BAUDCTRLA = RS232_RELOAD_BAUDRATE & 0xFF;
  USART.BAUDCTRLB = (RS232_RELOAD_BSCALE & 0xFF) | (RS232_RELOAD_BAUDRATE >> 8);

  /* Enable Low-Level RX Interrupt */
  USART.CTRLA = (USART.CTRLA & ~USART_RXCINTLVL_gm) | USART_RXCINTLVL_LO_gc;

  /* Enable Low-Level TX Interrupt */
  USART.CTRLA = (USART.CTRLA & ~USART_TXCINTLVL_gm) | USART_TXCINTLVL_LO_gc;

  /* Enable transmitter and receiver */
  USART.CTRLB |= USART_RXEN_bm;
  USART.CTRLB |= USART_TXEN_bm;
}


/* Status if sending buffer is empty
 * Return = status */
bool IsTxEmpty_b(void) {
  if (HeadTx_u8 != TailTx_u8)
    return false;
  else
    return true;
}

/* How many chars are in the sending buffer
 * Return = Number of available character */
u8 SizeTxBuffer_u8(void) {
  if (HeadTx_u8 == TailTx_u8)
    return false;
  else if (HeadTx_u8 > TailTx_u8)
    return HeadTx_u8 - TailTx_u8;
  else /* if (ucHeadTx < ucTailTx) */
    return MAX_LENGTH_TXBUF - TailTx_u8 + HeadTx_u8;
}

/* Write char into sending buffer
 * ucVal = Character */
void SetTxVal_v(u8 ucVal) {
  /* Disable TX Low-Level Interrupt */
  USART.CTRLA = USART.CTRLA & ~USART_TXCINTLVL_gm;

  TxBuf_u8[HeadTx_u8] = ucVal;
  HeadTx_u8++;

  if (HeadTx_u8 == MAX_LENGTH_TXBUF)
    HeadTx_u8 = 0;

  if (HeadTx_u8 == TailTx_u8)
    OverrunTx_b = 1;

  /* Enable TX Low-Level Interrupt */
  USART.CTRLA = (USART.CTRLA & ~USART_TXCINTLVL_gm) | USART_TXCINTLVL_LO_gc;
}

/* Get character from sending buffer
 * Return = character */
u8 GetTxVal_u8(void) {
  u8 RetVal_u8;

  RetVal_u8 = TxBuf_u8[TailTx_u8];
  TailTx_u8++;

  if (TailTx_u8 == MAX_LENGTH_TXBUF)
    TailTx_u8 = 0;

  return RetVal_u8;
}

/* Transmit character over RS232
 * Char_u8 = Character */
void Send_RS232_v(u8 Char_u8)  {
  /* Start new line */
  if (Char_u8 == '\n') {
    /* 0 = CrLf, 1 = Cr, 2 = Lf */
    switch (Newline_u8) {
      case 1:
      SetTxVal_v(CR);
      break;

      case 2:
      SetTxVal_v(LF);
      break;

      default:
      SetTxVal_v(CR);
      SetTxVal_v(LF);
    }
  }
  else
    SetTxVal_v(Char_u8);

  if (TxInProgress_b == 0) {
    /* Start transmitting */
    USART.DATA = GetTxVal_u8();

    /* Signal that transmitting is in progress */
    TxInProgress_b = 1;
  }
}

/* Transmit text array
 * Text_pu8 = Pointer to text array */
void Print_RS232_v(u8 * Text_pu8) {
  u8 Temp_u8;
  u8 Zaehler_u8 = 0;

  Temp_u8 = *Text_pu8;

  while (Temp_u8 != 0) {
    /* Repeat until end of array reached */
    Send_RS232_v(Temp_u8);
    Zaehler_u8++;
    Temp_u8 = *(Text_pu8 + Zaehler_u8);
  }
}

/* Status if receiving buffer is empty
 * Return = status */
bool IsRxEmpty_b(void) {
  if (HeadRx_u8 != TailRx_u8)
    return false;
  else
    return true;
}

/* How many chars are in the receiving buffer
 * Return = Number of available character */
u8 SizeRxBuffer_u8(void) {
  if (HeadRx_u8 == TailRx_u8)
    return false;
  else if (HeadRx_u8 > TailRx_u8)
    return HeadRx_u8 - TailRx_u8;
  else /*if (ucHeadRx < ucTailRx) */
    return MAX_LENGTH_RXBUF - TailRx_u8 + HeadRx_u8;
}

/* Write char into receiving buffer
 * ucVal = character */
void SetRxVal_v(u8 ucVal) {
  /* Disable RX Low-Level Interrupt */
  USART.CTRLA = USART.CTRLA & ~USART_RXCINTLVL_gm;

  /* Special handling for return */
  if (ucVal == '\n') {
    ModeChange_b = 1;

#ifdef ECHO
	/* Echo */
	Send_RS232_v('\n');
#endif
	}
	else {
	RxBuf_u8[HeadRx_u8] = ucVal;
	HeadRx_u8++;

#ifdef ECHO
	/* Echo */
	Send_RS232_v(ucVal);
#endif

  }

  if (HeadRx_u8 == MAX_LENGTH_RXBUF)
    HeadRx_u8 = 0;

  if (HeadRx_u8 == TailRx_u8)
    OverrunRx_b = 1;

  /* Enable RX Low-Level Interrupt */
  USART.CTRLA = (USART.CTRLA & ~USART_RXCINTLVL_gm) | USART_RXCINTLVL_LO_gc;
}

/* Get character from receiving buffer
 * Return = character */
u8 GetRxVal_u8(void) {
  u8 RetVal_u8;

  RetVal_u8 = RxBuf_u8[TailRx_u8];
  TailRx_u8++;

  if (TailRx_u8 == MAX_LENGTH_RXBUF)
    TailRx_u8 = 0;

  return RetVal_u8;
}

/******************************************************** Interruptfunctions ********************************************************/

/* Receive complete interrupt service routine */
ISR(USARTC0_RXC_vect)
{
  SetRxVal_v(USART.DATA);
}


/* Transmit complete interrupt service routine */
ISR(USARTC0_TXC_vect)
{
  if (!IsTxEmpty_b())
    USART.DATA = GetTxVal_u8();
  else
    TxInProgress_b = 0;
}
