#ifndef RS232_FUNCTIONS_H
  #define RS232_FUNCTIONS_H

  #include <avr/io.h>

  #define USART_PORT PORTC    /* USART on port C */
  #define USART USARTC0       /* Use usart0 */

  #define MAX_LENGTH_TXBUF 30	/* Send buffer size (255 Bytes max) */
  #define MAX_LENGTH_RXBUF 50	/* Receive buffer size (255 Bytes max) */

  #define CR 0x0D				/* Carriage Return */
  #define LF 0x0A				/* Line Feed */

  /* Select line termination */
  #define NEWLINE_CRLF 0
  #define NEWLINE_CR 1
  #define NEWLINE_LF 2

  //#define ECHO 1 /* Activate echo */

  #define RS232_BAUDRATE 115200UL
  #define RS232_BSCALE -7
  #define RS232_RELOAD_BSCALE ((RS232_BSCALE & 0x0F) << 4)

  /* 2^RS232_BSCALE, Don't know why the calculation doesn't work when in formula, so here defined as constant */
  #define RS232_BSCALE_EXP 0.0078125

  /* Only valid for BSCALE >= 0 */
//  #define RS232_RELOAD_BAUDRATE (u16) (F_CPU / (RS232_BSCALE_EXP * 16.0 * RS232_BAUDRATE) - 1)
//  #define RS232_REAL_BAUDRATE (F_CPU/(16.0 * RS232_BSCALE_EXP * (RS232_RELOAD_BAUDRATE - 1)))

  /* Only valid for BSCALE < 0 */
  #define RS232_RELOAD_BAUDRATE (u16) (((F_CPU / (16.0 * RS232_BAUDRATE)) - 1) / RS232_BSCALE_EXP)
  #define RS232_REAL_BAUDRATE (F_CPU/(16.0 * (RS232_BSCALE_EXP * RS232_RELOAD_BAUDRATE + 1)))

  #define BAUD_ERROR ((RS232_REAL_BAUDRATE * 100) / RS232_BAUDRATE)

  bool ModeChange_b;
  void InitRS232_v(void);
  bool IsTxEmpty_b(void);
  u8 SizeTxBuffer_u8(void);
  void SetTxVal_v(u8 ucVal);
  u8 GetTxVal_u8(void);
  void Send_RS232_v(u8 c);
  void Print_RS232_v(u8 * Text_p);
  bool IsRxEmpty_b(void);
  u8 SizeRxBuffer_u8(void);
  void SetRxVal_v(u8 ucVal);
  u8 GetRxVal_u8(void);
#endif
