#include <main.h>
#include <SHT21_Functions.h>
#include <I2C_Functions.h>

bool InitSHT21_b(void)
{
  /* 15ms delay to be sure the sensor is initialized */
  _delay_ms(15);

  if (SHT21WriteUserReg_b(false, true, SHT21_RESOLUTION_12_14))
  {
    return true;
  }

  return false;
}

/* Read temperature or humidity data from sensor
 * TypeOfData_u8 = Type of data */
f32 SHT21ReadData_f32(u8 TypeOfData_u8)
{
  u16 SHT21RawValue_u16;
  u8 Data_u8[3];

  /* Read data */
  if (!WriteI2C_b(SHT21_SLAVE_ADDRESS, 1, &TypeOfData_u8))
  {
    /* Something went wrong */
    return 0;
  }

  if (!ReadI2C_b(SHT21_SLAVE_ADDRESS, 3, Data_u8))
  {
    /* Something went wrong */
    return 0;
  }

  SHT21RawValue_u16 = ((Data_u8[0] << 8) + Data_u8[1]) & 0xFFFC;

  if (TypeOfData_u8 == SHT21_T_HOLD)
  {
    /* Calculate temperature */
    return (f32) SHT21RawValue_u16 * 175.72 / 65536 - 46.85;
  }
  else
  {
    /* Calculate humidity over water */
    return (f32) SHT21RawValue_u16 * 125 / 65536 - 6;
  }
}

/* Read user register content */
u8 SHT21ReadUserReg_u8(void)
{
  u8 Data_u8;

  /* Read data */
  Data_u8 = SHT21_USER_REG_R;
  if (!WriteI2C_b(SHT21_SLAVE_ADDRESS, 1, &Data_u8))
  {
    /* Something went wrong */
    return 0;
  }

  if (!ReadI2C_b(SHT21_SLAVE_ADDRESS, 1, &Data_u8))
  {
    /* Something went wrong */
    return 0;
  }

  return Data_u8;
}

bool SHT21WriteUserReg_b(bool HeaterOn_b, bool DisableOTP_b, u8 Resolution_u8)
{
  u8 Data_u8[2];
  Data_u8[0] = SHT21_USER_REG_W;

  Data_u8[1] = SHT21ReadUserReg_u8();

  if (HeaterOn_b)
  {
    Data_u8[1] |= SHT21_HEATER_gc;
  }
  else
  {
    Data_u8[1] &= ~SHT21_HEATER_gc;
  }

  if (DisableOTP_b)
  {
    Data_u8[1] |= SHT21_DISABLE_OTP_gc;
  }
  else
  {
    Data_u8[1] &= ~SHT21_DISABLE_OTP_gc;
  }

  switch (Resolution_u8)
  {
    case SHT21_RESOLUTION_12_14:
      Data_u8[1] &= 0x7E;
    break;
    case SHT21_RESOLUTION_08_12:
      Data_u8[1] &= 0x7F;
      Data_u8[1] |= 0x01;
    break;
    case SHT21_RESOLUTION_10_13:
      Data_u8[1] &= 0xFE;
      Data_u8[1] |= 0x80;
    break;
    case SHT21_RESOLUTION_11_11:
      Data_u8[1] |= 0x81;
    break;
    default:
      /* Default to max resolution */
      Data_u8[1] &= 0x7E;
    break;
  }

  if (WriteI2C_b(SHT21_SLAVE_ADDRESS, 2, Data_u8))
  {
    return true;
  }

  /* Something went wrong */
  return false;
}