#include <main.h>
#include <RS232_Functions.h>
#include <I2C_Functions.h>
#include <SHT21_Functions.h>

void Init_v(void);

void Init_v(void)
{
  InitRS232_v();
  InitI2C_v();

  /* Enable all Low-Level Interrupt */
  PMIC.CTRL |= PMIC_LOLVLEX_bm;

  /* Enable global interrupts. */
  sei();
}

int main(void)
{
  u8 Text_u8[] = {"I2C SHT21 Test\n"};
  u8 UserReg_u8;
  u8 Ascii_u8[15];
  f32 SHT21CalculatedT_f32;
  f32 SHT21CalculatedRHwater_f32;
  f32 SHT21CalculatedRHice_f32;

  Init_v();

  Print_RS232_v(Text_u8);

  /* 50ms delay to be sure the sensor is initialized */
  _delay_ms(50);

  SHT21WriteUserReg_v(false, true, SHT21_RESOLUTION_12_14);

  while(1)
  {
    while(!IsTxEmpty_b());

    UserReg_u8 = SHT21ReadUserReg_u8();

    /* Convert 16 bit value to ASCII hex */
    utoa(UserReg_u8, (char*) Ascii_u8, 16);

    Send_RS232_v('U');
    Send_RS232_v('=');
    Send_RS232_v('0');
    Send_RS232_v('x');
    Print_RS232_v(Ascii_u8);
    Send_RS232_v('\n');

    /* Calculated temperature */
    SHT21CalculatedT_f32 = SHT21ReadData_f32(SHT21_T_HOLD);

    /* Calculated humidity over water */
    SHT21CalculatedRHwater_f32 = SHT21ReadData_f32(SHT21_RH_HOLD);

    /* Calculate humidity over ice */
    SHT21CalculatedRHice_f32 = SHT21CalculatedRHwater_f32 * exp(BETA_W * SHT21CalculatedT_f32 / (LAMBDA_W + SHT21CalculatedT_f32)) / exp(BETA_I * SHT21CalculatedT_f32 / (LAMBDA_I + SHT21CalculatedT_f32));

    /* Convert double to ASCII (this function alone uses 3,6kb code size!!!) */
    dtostre(SHT21CalculatedT_f32, (char*) Ascii_u8, 3, DTOSTR_PLUS_SIGN);

    Send_RS232_v('T');
    Send_RS232_v('=');
    Print_RS232_v(Ascii_u8);
    Send_RS232_v('\n');

    /* Convert double to ASCII (this function alone uses 3,6kb code size!!!) */
    dtostre(SHT21CalculatedRHwater_f32, (char*) Ascii_u8, 3, DTOSTR_PLUS_SIGN);

    Send_RS232_v('%');
    Send_RS232_v('=');
    Print_RS232_v(Ascii_u8);
    Send_RS232_v('\n');

    /* Convert double to ASCII (this function alone uses 3,6kb code size!!!) */
    dtostre(SHT21CalculatedRHice_f32, (char*) Ascii_u8, 3, DTOSTR_PLUS_SIGN);

    Send_RS232_v('%');
    Send_RS232_v('=');
    Print_RS232_v(Ascii_u8);
    Send_RS232_v('\n');

    _delay_ms(2000);
  }
}

